"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$template = exports.$comment = exports.$text = exports.VERSION = void 0;
const pierrejs_1 = require("pierrejs");
const expression_1 = require("./expression");
const utils_1 = require("./utils");
const topOfStack = (stack) => stack[stack.length - 1];
const topOfStackStmts = (stack) => {
    const top = stack[stack.length - 1];
    if ('elseStatements' in top) {
        return top.elseStatements;
    }
    return top.statements;
};
const buildBlock = (loc, expression, isNegated) => ({
    type: 'BLOCK',
    loc,
    isNegated,
    expression,
    statements: [],
});
exports.VERSION = 2;
exports.$text = utils_1.text
    .map((value, loc) => ({ type: 'TEXT', loc, value }))
    .withName('text');
exports.$comment = pierrejs_1.default.all(utils_1.char, utils_1.text)
    .map((0, utils_1.atPos)(1))
    .map((value, { start, end }) => ({ type: 'COMMENT', loc: { start: start - 2, end: end + 2 }, value }))
    .withName('comment');
const $mustache = expression_1.$expression.map((expression, { start, end }) => ({
    type: 'MUSTACHE',
    loc: { start: start - 2, end: end + 2 },
    expression,
}));
exports.$template = pierrejs_1.default.context('mustache', function* () {
    const stack = [{
            type: 'TEMPLATE',
            version: exports.VERSION,
            statements: [],
        }];
    /* $lab:coverage:off$ */
    while (true) {
        /* $lab:coverage:on$ */
        const txt = yield pierrejs_1.default.optional(exports.$text);
        if (txt) {
            topOfStackStmts(stack).push(txt);
            // no need to `continue`, two texts in a row aren't possible
        }
        const open = yield pierrejs_1.default.optional(utils_1.openMustache).map((v, loc) => v ? loc : null);
        if (open) {
            switch (yield utils_1.peek) {
                case '!': {
                    topOfStackStmts(stack).push(yield exports.$comment);
                    break;
                }
                case '#':
                case '^': {
                    const typeChar = yield utils_1.char;
                    const expression = yield expression_1.$expression;
                    if (expression.type === 'LITERAL') {
                        yield pierrejs_1.default.fail(`Blocks must receive path expressions or helpers. Literal blocks are not allowed.`);
                        // Never happens, just for typescript to know that below here, expression is not LiteralStatement
                        /* $lab:coverage:off$ */
                        return;
                        /* $lab:coverage:on$ */
                    }
                    stack.push(buildBlock(open, expression, typeChar === '^'));
                    break;
                }
                case '/': {
                    yield utils_1.char; // Consuming '/'
                    const expression = yield expression_1.$expression;
                    if (expression.type === 'LITERAL') {
                        yield pierrejs_1.default.fail(`Unexpected {{/${expression.value}}}. Literal blocks are not allowed.`);
                        // Never happens, just for typescript to know that below here, expression is not LiteralStatement
                        /* $lab:coverage:off$ */
                        return;
                        /* $lab:coverage:on$ */
                    }
                    if (expression.params.length > 0) {
                        yield pierrejs_1.default.fail(`Closing blocks cannot have parameters`);
                    }
                    const name = expression.path;
                    if (stack.length <= 1) {
                        yield pierrejs_1.default.fail(`Unexpected {{/${name}}}, this block wasn't opened`);
                    }
                    const block = stack.pop();
                    if (block.expression.path !== name) {
                        yield pierrejs_1.default.fail(`Unexpected {{/${name}}}, this block was opened as {{#${block.expression.path}}}`);
                    }
                    block.loc.end = expression.loc.end + 2;
                    topOfStackStmts(stack).push(block);
                    break;
                }
                default: {
                    const stmt = yield $mustache;
                    // Normal mustache
                    if (stmt.expression.type !== 'EXPRESSION' || stmt.expression.path !== 'else') {
                        topOfStackStmts(stack).push(stmt);
                        break;
                    }
                    // Else block
                    if (stmt.expression.params.length > 0) {
                        yield pierrejs_1.default.fail('{{else}} blocks cannot have parameters');
                    }
                    if (stack.length <= 1) {
                        yield pierrejs_1.default.fail('{{else}} can only exist inside blocks');
                    }
                    const top = topOfStack(stack);
                    if (Array.isArray(top.elseStatements)) {
                        yield pierrejs_1.default.fail(`an {{else}} block was already defined for the block ${top.expression.path}`);
                    }
                    top.elseStatements = [];
                    break;
                }
            }
            yield utils_1.optionalSpaces;
            yield utils_1.closeMustache;
            continue;
        }
        const end = yield pierrejs_1.default.optional(pierrejs_1.default.end());
        if (end) {
            break;
        }
        yield pierrejs_1.default.fail('Unexpected end of file');
    }
    if (stack.length > 1) {
        const block = topOfStack(stack);
        yield pierrejs_1.default.fail(`Expected {{/${block.expression.path}}}, make sure this block was closed`);
    }
    return stack[0];
}).map(({ type, ...v }, loc) => ({ type, loc, ...v }));
