"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.lString = void 0;
/* eslint-disable no-unused-vars */
const pierrejs_1 = require("pierrejs");
const utils_1 = require("./utils");
/* $lab:coverage:off$ */
var State;
(function (State) {
    State[State["READ_CHAR"] = 0] = "READ_CHAR";
    State[State["READ_ESCAPED_CHAR"] = 1] = "READ_ESCAPED_CHAR";
})(State || (State = {}));
/* $lab:coverage:on$ */
const escapes = {
    '"': '"',
    '\'': '\'',
    '`': '`',
    '\\': '\\',
    'n': '\n',
    'r': '\r',
    't': '\t',
};
const doubleQuote = pierrejs_1.default.string('"').withName('double quotes');
const singleQuote = pierrejs_1.default.string('\'').withName('single quote');
const graveAccent = pierrejs_1.default.string('`').withName('grave accent');
const quote = pierrejs_1.default.oneOf(doubleQuote, singleQuote, graveAccent);
exports.lString = pierrejs_1.default.context('string', function* () {
    const openQuote = yield quote;
    let state = State.READ_CHAR;
    let content = '';
    /* $lab:coverage:off$ */
    while (true) {
        /* $lab:coverage:on$ */
        switch (state) {
            case State.READ_CHAR: {
                const c = yield utils_1.char;
                if (c === '\\') {
                    state = State.READ_ESCAPED_CHAR;
                    break;
                }
                if (c === openQuote) {
                    return content;
                }
                content += c;
                break;
            }
            case State.READ_ESCAPED_CHAR: {
                const c = yield utils_1.char;
                content += escapes[c] || c;
                state = State.READ_CHAR;
                break;
            }
            /* $lab:coverage:off$ */
            default:
                yield pierrejs_1.default.fail(`Unexpected state ${state} at string parser`);
            /* $lab:coverage:on$ */
        }
    }
});
