"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runBlock = void 0;
const _1 = require(".");
async function runBlock(execution, block) {
    const value = await (0, _1.runStatement)(execution, block.expression);
    // Negated blocks
    if (block.isNegated) {
        // Value is true and there is an else block
        if (value && Array.isArray(block.elseStatements)) {
            return await (0, _1.runStatements)(execution, block.elseStatements);
        }
        // Value is true and there is no else block
        if (value) {
            return null;
        }
        // Value is false
        return await (0, _1.runStatements)(execution, block.statements);
    }
    // Falsy value or empty array
    if (!value || (Array.isArray(value) && value.length === 0)) {
        // Value is false and there is an else block
        if (Array.isArray(block.elseStatements)) {
            return await (0, _1.runStatements)(execution, block.elseStatements);
        }
        // Value is false and there is no else block
        return null;
    }
    // Non empty array
    if (Array.isArray(value)) {
        let result = '';
        for (const item of value) {
            result += await (0, _1.runStatements)(execution.withChildContext(item), block.statements);
        }
        return result;
    }
    // Object
    if (typeof value === 'object') {
        return await (0, _1.runStatements)(execution.withChildContext(value), block.statements);
    }
    // Truthy value
    return await (0, _1.runStatements)(execution, block.statements);
}
exports.runBlock = runBlock;
